@@ -16,7 +16,6 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#define _GNU_SOURCE
 #define _LGPL_SOURCE
 #include <errno.h>
 #include <limits.h>
@@ -31,12 +30,14 @@
 #include <sched.h>
 #include <sys/signal.h>
 #include <assert.h>
-#include <sys/prctl.h>
+#include <signal.h>
 
 #include <common/common.h>
 #include <common/utils.h>
 #include <common/compat/getenv.h>
-#include <common/sessiond-comm/unix.h>
+#include <common/compat/prctl.h>
+#include <common/unix.h>
+#include <common/defaults.h>
 
 #include "runas.h"
 
@@ -170,7 +171,7 @@ run_as_fct run_as_enum_to_fct(enum run_as_cmd cmd)
 	case RUN_AS_MKDIR_RECURSIVE:
 		return _mkdir_recursive;
 	default:
-		ERR("Unknown command %d", (int) cmd)
+		ERR("Unknown command %d", (int) cmd);
 		return NULL;
 	}
 }
@@ -323,8 +324,9 @@ int run_as_worker(struct run_as_worker *worker)
 	memset(worker->procname, 0, proc_orig_len);
 	strncpy(worker->procname, DEFAULT_RUN_AS_WORKER_NAME, proc_orig_len);
 
-	ret = prctl(PR_SET_NAME, DEFAULT_RUN_AS_WORKER_NAME, 0, 0, 0);
-	if (ret) {
+	ret = lttng_prctl(PR_SET_NAME,
+			(unsigned long) DEFAULT_RUN_AS_WORKER_NAME, 0, 0, 0);
+	if (ret && ret != -ENOSYS) {
 		/* Don't fail as this is not essential. */
 		PERROR("prctl PR_SET_NAME");
 		ret = 0;
@@ -373,7 +375,7 @@ int run_as_cmd(struct run_as_worker *worker,
 			recvret.ret = -1;
 			recvret._errno = EPERM;
 			ERR("Client (%d)/Server (%d) UID mismatch (and sessiond is not root)",
-				uid, geteuid());
+				(int) uid, (int) geteuid());
 			goto end;
 		}
 	}
@@ -465,7 +467,7 @@ int run_as_mkdir_recursive(const char *path, mode_t mode, uid_t uid, gid_t gid)
 	struct run_as_data data;
 
 	DBG3("mkdir() recursive %s with mode %d for uid %d and gid %d",
-			path, mode, uid, gid);
+			path, (int) mode, (int) uid, (int) gid);
 	strncpy(data.u.mkdir.path, path, PATH_MAX - 1);
 	data.u.mkdir.path[PATH_MAX - 1] = '\0';
 	data.u.mkdir.mode = mode;
@@ -478,7 +480,7 @@ int run_as_mkdir(const char *path, mode_t mode, uid_t uid, gid_t gid)
 	struct run_as_data data;
 
 	DBG3("mkdir() %s with mode %d for uid %d and gid %d",
-			path, mode, uid, gid);
+			path, (int) mode, (int) uid, (int) gid);
 	strncpy(data.u.mkdir.path, path, PATH_MAX - 1);
 	data.u.mkdir.path[PATH_MAX - 1] = '\0';
 	data.u.mkdir.mode = mode;
@@ -495,7 +497,7 @@ int run_as_open(const char *path, int flags, mode_t mode, uid_t uid, gid_t gid)
 	struct run_as_data data;
 
 	DBG3("open() %s with flags %X mode %d for uid %d and gid %d",
-			path, flags, mode, uid, gid);
+			path, flags, (int) mode, (int) uid, (int) gid);
 	strncpy(data.u.open.path, path, PATH_MAX - 1);
 	data.u.open.path[PATH_MAX - 1] = '\0';
 	data.u.open.flags = flags;
@@ -509,7 +511,7 @@ int run_as_unlink(const char *path, uid_t uid, gid_t gid)
 	struct run_as_data data;
 
 	DBG3("unlink() %s with for uid %d and gid %d",
-			path, uid, gid);
+			path, (int) uid, (int) gid);
 	strncpy(data.u.unlink.path, path, PATH_MAX - 1);
 	data.u.unlink.path[PATH_MAX - 1] = '\0';
 	return run_as(RUN_AS_UNLINK, &data, uid, gid);
@@ -521,7 +523,7 @@ int run_as_rmdir_recursive(const char *path, uid_t uid, gid_t gid)
 	struct run_as_data data;
 
 	DBG3("rmdir_recursive() %s with for uid %d and gid %d",
-			path, uid, gid);
+			path, (int) uid, (int) gid);
 	strncpy(data.u.rmdir_recursive.path, path, PATH_MAX - 1);
 	data.u.rmdir_recursive.path[PATH_MAX - 1] = '\0';
 	return run_as(RUN_AS_RMDIR_RECURSIVE, &data, uid, gid);
@@ -530,21 +532,13 @@ int run_as_rmdir_recursive(const char *path, uid_t uid, gid_t gid)
 static
 int reset_sighandler(void)
 {
-	int sig, ret = 0;
+	int sig;
 
-	for (sig = SIGHUP; sig <= SIGUNUSED; sig++) {
-		/* Skip unblockable signals. */
-		if (sig == SIGKILL || sig == SIGSTOP) {
-			continue;
-		}
-		if (signal(sig, SIG_DFL) == SIG_ERR) {
-			PERROR("reset signal %d", sig);
-			ret = -1;
-			goto end;
-		}
+	DBG("Resetting run_as worker signal handlers to default");
+	for (sig = 1; sig <= 31; sig++) {
+		(void) signal(sig, SIG_DFL);
 	}
-end:
-	return ret;
+	return 0;
 }
 
 static
@@ -659,6 +653,7 @@ int run_as_create_worker(char *procname)
 			ret = -1;
 		}
 		worker->sockpair[1] = -1;
+		LOG(ret ? PRINT_ERR : PRINT_DBG, "run_as worker exiting (ret = %d)", ret);
 		exit(ret ? EXIT_FAILURE : EXIT_SUCCESS);
 	} else {
 		/* Parent */
@@ -708,11 +703,13 @@ void run_as_destroy_worker(void)
 {
 	struct run_as_worker *worker = global_worker;
 
+	DBG("Destroying run_as worker");
 	pthread_mutex_lock(&worker_lock);
 	if (!worker) {
 		goto end;
 	}
 	/* Close unix socket */
+	DBG("Closing run_as worker socket");
 	if (lttcomm_close_unix_sock(worker->sockpair[0])) {
 		PERROR("close");
 	}
