@@ -15,7 +15,6 @@
  * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#define _GNU_SOURCE
 #define _LGPL_SOURCE
 #include <assert.h>
 #include <limits.h>
@@ -104,7 +103,7 @@ int lttcomm_bind_inet_sock(struct lttcomm_sock *sock)
 {
 	int ret;
 
-	ret = bind(sock->fd, &sock->sockaddr.addr.sin,
+	ret = bind(sock->fd, (const struct sockaddr *) &sock->sockaddr.addr.sin,
 			sizeof(sock->sockaddr.addr.sin));
 	if (ret < 0) {
 		PERROR("bind inet");
@@ -408,7 +407,7 @@ ssize_t lttcomm_recvmsg_inet_sock(struct lttcomm_sock *sock, void *buf,
  * Return the size of sent data.
  */
 LTTNG_HIDDEN
-ssize_t lttcomm_sendmsg_inet_sock(struct lttcomm_sock *sock, void *buf,
+ssize_t lttcomm_sendmsg_inet_sock(struct lttcomm_sock *sock, const void *buf,
 		size_t len, int flags)
 {
 	struct msghdr msg;
@@ -417,7 +416,7 @@ ssize_t lttcomm_sendmsg_inet_sock(struct lttcomm_sock *sock, void *buf,
 
 	memset(&msg, 0, sizeof(msg));
 
-	iov[0].iov_base = buf;
+	iov[0].iov_base = (void *) buf;
 	iov[0].iov_len = len;
 	msg.msg_iov = iov;
 	msg.msg_iovlen = 1;
