@@ -15,7 +15,6 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#define _GNU_SOURCE
 #define _LGPL_SOURCE
 #include <assert.h>
 #include <string.h>
@@ -24,7 +23,7 @@
 #include <lttng/load.h>
 #include <lttng/load-internal.h>
 #include <common/sessiond-comm/sessiond-comm.h>
-#include <common/config/config.h>
+#include <common/config/session-config.h>
 
 #include "lttng-ctl-helper.h"
 
@@ -84,7 +83,7 @@ int lttng_load_session_attr_set_session_name(
 		size_t len;
 
 		len = strlen(session_name);
-		if (len >= NAME_MAX) {
+		if (len >= LTTNG_NAME_MAX) {
 			ret = -LTTNG_ERR_INVALID;
 			goto error;
 		}
